#lang racket

(require text-table)

(struct hist (date open high low close adj-close vol)
  #:transparent)

(define (import-record port)
  (let ([rec (read-line port)])
    (if (eof-object? rec)
        eof
        (match (string-split rec ",")
          [(list date open high low close adj-close vol)
           (hist date
                 (string->number open)
                 (string->number high)
                 (string->number low)
                 (string->number close)
                 (string->number adj-close)
                 (string->number vol))]
          [_ (error "Failed to load record.")]))))

(define (format-rec rec width prec)
  (match rec
    [(hist date open high low close adj-close vol)
     (hist date
           (~r open #:min-width width  #:precision (list '= prec))
           (~r high #:min-width width  #:precision (list '= prec))
           (~r low #:min-width width  #:precision (list '= prec))
           (~r close #:min-width width  #:precision (list '= prec))
           (~r adj-close #:min-width width  #:precision (list '= prec))
           (~r vol #:min-width 9 ))]))

;change path as needed
(define (show sym period filter)
  (let ([in-port (open-input-file
                  (build-path "StockHistory" period (string-append sym ".csv")))])
    (read-line in-port) ; skip past header row
    (let* ([recs
            (reverse
             (let loop([rec-list '()])
               (let ([rec (import-record in-port)])
                 (if (eof-object? rec)
                     rec-list
                     (if (filter rec)
                         (let* ([rec (format-rec rec 8 2)]
                                [rec (list (hist-date rec)
                                           (hist-high rec)
                                           (hist-low rec)
                                           (hist-close rec))])                          
                           (loop (cons rec rec-list)))
                         (loop rec-list))))))]
           [tbl-list (cons (list "   Date" "  High" "  Low" "  Close") recs)])
      (close-input-port in-port)
      (display (table->string tbl-list)))))

(define (filt rec)
  (equal? (substring (hist-date rec) 0 4) "2008"))