#lang racket
(require racklog)

(define DIGITS 5)

(define (score c h)
  (let loop ([l1 c] [l2 h] [boles 0] [creots 0])
    (if (equal? l1 null)
        (cons boles creots)
        (let ([d1 (car l1)]
              [d2 (car l2)]
              [t1 (cdr l1)]
              [t2 (cdr l2)])
          (if (= d1 d2) 
              (loop t1 t2 (add1 boles) creots)
              (loop t1 t2 boles (+ creots (if (member d1 h) 1 0))))))))

(define (randomize-digits)
  (let loop([count 10] [l '()])
    (if (= count 0) l
        (let ([d (random 10)])
          (if (member d l)
              (loop count l)
              (loop (sub1 count) (cons d l)))))))

(define %permute-n
  (%rel (l h t u v w n m)
        [((_) '() 0) !]
        [(l (cons h t) n)
         (%append v (cons h u) l)
         (%append v u w)
         (%is m (sub1 n))
         (%permute-n w t m)]))

(define history '())

(define %update-history
  (%rel (guess boles creots)
        [(guess boles creots)
         (%is #t
              (begin
                (set! history (cons (list guess boles creots) history))
                #t))]))

(define %consistent?
  (%rel (g h hb hc gb gc t)
        [((_) '()) %true]
        [(g (cons (list h hb hc) t))
         (%is (cons gb gc) (score g h))
         (%and (%=:= hb gb) (%=:= hc gc))
         (%consistent? g t)]
        [(g) (%consistent? g history)]))

(define %print
  (%rel (fmt val)
        [(fmt val)
         (%is #t (begin (printf fmt val) #t))]))

(define (get-input)
  (let ([val (read (current-input-port))])
    (let-values ([(boles creots) (quotient/remainder val 10)])
      (cons boles creots))))

(define %repl
  (%rel (digits guess val boles creots)
        [()
         (%is digits (randomize-digits))
         (%is #t (begin (set! history '()) #t))
         (%repl digits)]
        [(digits)
         (%permute-n digits guess DIGITS)
         (%consistent? guess)
         (%print "\nGuess: ~a\n" guess)
         (%= (cons boles creots) (get-input))
         (%update-history guess boles creots)
         (%if-then-else (%=:= boles DIGITS) ! %fail)]))         